import MeCab

# Initialize mecab-python3
wakati = MeCab.Tagger("-Owakati")
tagger = MeCab.Tagger()


def tokenize(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """
    words = wakati.parse(text).split()
    return words


def original_usage(text):
    """
    Return the analysis results by mecab-python3.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : str
        The default analysis results by mecab-python3
    """
    tokens = tagger.parse(text)
    return tokens



from spacy.lang.ja import Japanese

# Load SudachiPy with split mode A (default)
nlp = Japanese()


def tokenize(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """
    doc = nlp.make_doc(text)
    words = [w.text for w in doc]
    return words


def original_usage(text):
    """
    Return the analysis results by spacy.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : spacy.tokens.doc.Doc
        The analysis results by spacy
    """
    tokens = nlp(text)
    return tokens



import os
import pathlib


base = pathlib.Path(os.path.abspath(__file__))
base = str(base.parent)

sample_train = os.path.join(base, "data", "sample.train")
sample_dev = os.path.join(base, "data", "sample.dev")
sample_test = os.path.join(base, "data", "sample.test")
sample_txt = os.path.join(base, "data", "sample.txt")



from . import sample_datasets


from .download import (
    download_corpus,
    available_corpus
)

from .preprocess import load_corpus

from .downloader_utils import (
    get_resource_dir,
    get_corpora_dict
)



import time
import pickle

from sklearn.svm import LinearSVC
from sklearn.metrics import f1_score
from sklearn.metrics import accuracy_score
from sklearn.metrics import classification_report
from sklearn.feature_extraction.text import TfidfVectorizer


from toiro import tokenizers
from toiro import classifiers


class SVMClassificationModel:
    """
    Text classification model based on SVM and tf-idf.

    Attributes
    ----------
    tokenizer : str
        The word segmenter for Japanese text.
        You can choose tokenizers in tokenizers.get_avaiable_tokenizers().

    original_tokenizer : func
        An original tokenizer of your own definition.

    model_file : str
        The trained model file

    Methods
    -------
    fit(train_df, dev_df=None)
        Train a text classification model.
    eval(eval_df)
        Evaluate the trained model.
    predict(text)
        Predict a label.
    save(filename)
        Save the trained model.
    """

    def __init__(self, tokenizer="janome",
                 original_tokenizer=None, model_file=None):
        self.tokenizer = tokenizers.SelectTokenizer(tokenizer)

        if original_tokenizer:
            self.tokenize = original_tokenizer
        else:
            self.tokenize = self.tokenizer.tokenize

        if model_file is None:
            self.tfidf_extracter = TfidfVectorizer(
                ngram_range=(1, 2),
                tokenizer=self.tokenize,
                max_df=0.9,
                min_df=3,
                sublinear_tf=1
            )
            self.svm = LinearSVC()
        else:
            with open(model_file, "rb") as f:
                self.svm, self.tfidf_extracter = pickle.load(f)

        self.elapsed_time = None

    def fit(self, train_df, dev_df=None):
        start = time.time()
        train_Y = train_df[0].values
        train_X = self.tfidf_extracter.fit_transform(train_df[1])
        self.svm.fit(train_X, train_Y)
        self.elapsed_time = time.time() - start

    def eval(self, eval_df):
        test_Y = eval_df[0].values
        test_X = self.tfidf_extracter.transform(eval_df[1])

        pred_Y = self.svm.predict(test_X)
        accuracy = accuracy_score(test_Y, pred_Y)
        macro_f1 = f1_score(test_Y, pred_Y, average="macro")
        cr = classification_report(test_Y, pred_Y)
        eval_metrics = classifiers.EvaluationMetrics(
            accuracy, macro_f1, cr, self.elapsed_time
        )
        return eval_metrics

    def predict(self, text):
        text = [text]
        x = self.tfidf_extracter.transform(text)
        pred_y = self.svm.predict(x)
        pred_y = pred_y[0]
        return pred_y

    def save(self, filename):
        with open(filename, "wb") as f:
            pickle.dump((self.svm, self.tfidf_extracter), f)



import pandas as pd


def read_file(filename):
    data = pd.read_csv(filename, delimiter='\t', header=None, dtype={0: str})
    return data


class EvaluationMetrics:

    def __init__(self, accuracy_score, macro_f1_score, classification_report,
                 elapsed_time=None):
        self.metrics = {
            "accuracy_score": accuracy_score,
            "macro_f1_score": macro_f1_score,
            "classification_report": classification_report,
            "elapsed_time": elapsed_time
        }

    def __getitem__(self, key):
        return self.metrics[key]

    def __str__(self):
        output = []
        for k, v in self.metrics.items():
            output.append(f"{k}: {v}")
        return "\n".join(output)



from .tokenizer_utils import (
    is_nagisa_available,
    is_janome_available,
    is_mecab_available,
    is_sudachipy_available,
    is_spacy_available,
    is_ginza_available,
    is_kytea_available,
    is_jumanpp_available,
    is_sentencepiece_available,
    is_fugashi_ipadic_available,
    is_tinysegmenter_available,
    is_fugashi_unidic_available,
    is_tiktoken_available,
    available_tokenizers
)


if is_nagisa_available():
    from .tokenizer_nagisa import tokenize as tokenize_nagisa
    from .tokenizer_nagisa import original_usage as original_nagisa


if is_janome_available():
    from .tokenizer_janome import tokenize as tokenize_janome
    from .tokenizer_janome import original_usage as original_janome


if is_mecab_available():
    from .tokenizer_mecab_python3 import tokenize as tokenize_mecab
    from .tokenizer_mecab_python3 import original_usage as original_mecab


if is_sudachipy_available():
    from .tokenizer_sudachipy import tokenize as tokenize_sudachipy
    from .tokenizer_sudachipy import original_usage as original_sudachipy


if is_spacy_available():
    from .tokenizer_spacy import tokenize as tokenize_spacy
    from .tokenizer_spacy import original_usage as original_spacy


if is_ginza_available():
    from .tokenizer_ginza import tokenize as tokenize_ginza
    from .tokenizer_ginza import original_usage as original_ginza


if is_kytea_available():
    from .tokenizer_kytea import tokenize as tokenize_kytea
    from .tokenizer_kytea import original_usage as original_kytea


if is_jumanpp_available():
    from .tokenizer_jumanpp import tokenize as tokenize_jumanpp
    from .tokenizer_jumanpp import original_usage as original_jumanpp


if is_sentencepiece_available():
    from .tokenizer_sentencepiece import tokenize as tokenize_sentencepiece
    from .tokenizer_sentencepiece import original_usage as original_sentencepiece


if is_fugashi_ipadic_available():
    from .tokenizer_fugashi_ipadic import tokenize as tokenize_fugashi_ipadic
    from .tokenizer_fugashi_ipadic import original_usage as original_fugashi_ipadic


if is_tinysegmenter_available():
    from .tokenizer_tinysegmenter import tokenize as tokenize_tinysegmenter
    from .tokenizer_tinysegmenter import original_usage as original_tinysegmenter


if is_fugashi_unidic_available():
    from .tokenizer_fugashi_unidic import tokenize as tokenize_fugashi_unidic
    from .tokenizer_fugashi_unidic import original_usage as original_fugashi_unidic

if is_tiktoken_available():
    from .tokenizer_tiktoken import tokenize_gpt4o as tokenize_tiktoken_gpt4o
    from .tokenizer_tiktoken import original_usage_gpt4o as original_tiktoken_gpt4o
    from .tokenizer_tiktoken import tokenize_gpt5 as tokenize_tiktoken_gpt5
    from .tokenizer_tiktoken import original_usage_gpt5 as original_tiktoken_gpt5


from .tokenizer_report import (
    compare,
    compare_from_file,
    print_words,
    SelectTokenizer,
    get_avaiable_tokenizers
)



import nagisa


def tokenize(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """
    words = nagisa.wakati(text)
    return words


def original_usage(text):
    """
    Return the analysis results by nagisa.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : nagisa.tagger.Tagger._Token
        The analysis results by nagisa
    """
    tokens = nagisa.tagging(text)
    return tokens



from janome.tokenizer import Tokenizer

# Initialize Janome
t = Tokenizer()


def tokenize(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """
    words = list(t.tokenize(text, wakati=True))
    return words


def original_usage(text):
    """
    Return the analysis results by Janome.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : list
        The analysis results by Janome
    """
    tokens = list(t.tokenize(text))
    return tokens



import tiktoken

enc_gpt4o = tiktoken.encoding_for_model("gpt-4o")
enc_gpt5 = tiktoken.encoding_for_model("gpt-5")


def token_ids_to_tokens(token_ids, enc):
    tokens = []
    for token_id in token_ids:
        try:
            token_bytes = enc.decode_bytes([token_id])
            token_str = token_bytes.decode("utf-8", errors="replace")
        except Exception:
            token_str = "<INVALID>"
        tokens.append(token_str)
    return tokens


def tokenize_gpt4o(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """
    token_ids = enc_gpt4o.encode(text)
    tokens = token_ids_to_tokens(token_ids, enc_gpt4o)
    return tokens


def original_usage_gpt4o(text):
    """
    Return the analysis results by tiktoken for gpt-4o.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : list
        A list of words
    """
    tokens = tokenize_gpt4o(text)
    return tokens


def tokenize_gpt5(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """
    token_ids = enc_gpt5.encode(text)
    tokens = token_ids_to_tokens(token_ids, enc_gpt5)
    return tokens


def original_usage_gpt5(text):
    """
    Return the analysis results by tiktoken for gpt-5.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : list
        A list of words
    """
    tokens = tokenize_gpt5(text)
    return tokens



from sudachipy import tokenizer
from sudachipy import dictionary

# Initialize sudachipy
tokenizer_obj = dictionary.Dictionary().create()
mode = tokenizer.Tokenizer.SplitMode.C


def tokenize(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """
    words = [m.surface() for m in tokenizer_obj.tokenize(text, mode)]
    return words


def original_usage(text):
    """
    Return the analysis results by SudachiPy.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : sudachipy.morphemelist.MorphemeList
        The analysis results by SudachiPy
    """
    tokens = tokenizer_obj.tokenize(text, mode)
    return tokens



import os
import requests
from pathlib import Path

from tqdm import tqdm

from .downloader_utils import get_resource_dir
from .downloader_utils import get_corpora_dict


def available_corpus():
    """
    Return a list of corpora available in this tool.

    Returns
    -------
    corpora_list : list
        The available corpus list

    Examples
    --------
    >>> toiro.available_corpus()
    ['livedoor_news_corpus', 'yahoo_movie_reviews', 'amazon_reviews']

    """
    corpora_dict = get_corpora_dict()
    corpora_list = list(corpora_dict.keys())
    return corpora_list


def download_corpus(corpus='yahoo_movie_reviews'):
    """
    Download the corpus from the download link.

    This module is used to download the corpus of the list
    ['livedoor_news_corpus', 'yahoo_movie_reviews', 'amazon_reviews'].

    Parameters
    ----------
    corpus : str
        The corpus to download


    Examples
    --------
    >>> toiro.download_corpus(corpus='yahoo_movie_reviews')

    """
    corpora_dict = get_corpora_dict()

    corpora_list = list(corpora_dict.keys())

    if corpus not in corpora_list:
        err_msg = f"{corpus} is not available. Choose from {corpora_list}"
        raise Exception(err_msg)

    resource_dir = get_resource_dir()

    Path(resource_dir).mkdir(parents=True, exist_ok=True)

    filename = corpora_dict[corpus]["filename"]
    filepath = os.path.join(resource_dir, filename)
    download_url = corpora_dict[corpus]["url"]

    # download a corpus
    if os.path.exists(filepath):
        msg = f"{corpus} ({filename}) exists in {resource_dir} ."
        print(msg)
    else:
        verbose = True
        r = requests.get(download_url, stream=True)
        with open(filepath, 'wb') as f:
            file_size = int(r.headers.get('content-length'))
            default_chunk_size = 13107
            desc = f'Downloading {filename} from {download_url}'

            with tqdm(total=file_size, unit='B', unit_scale=True,
                      disable=not verbose, desc=desc) as pbar:

                for chunk in r.iter_content(chunk_size=default_chunk_size):
                    if chunk:
                        f.write(chunk)
                        f.flush()
                        pbar.update(len(chunk))



from pyknp import Juman

jumanpp = Juman()


def tokenize(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """

    # Error avoidance for long text
    text = text[:1024]

    # Error avoidance for space
    text = text.replace(' ', '　')

    result = jumanpp.analysis(text)
    words = [mrph.midasi for mrph in result.mrph_list()]
    return words


def original_usage(text):
    """
    Return the analysis results by Juman++.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : pyknp.juman.mlist.MList
        The analysis results by Juman++
    """

    # Error avoidance for long text
    text = text[:1024]

    # Error avoidance for space
    text = text.replace(' ', '　')

    tokens = jumanpp.analysis(text)
    return tokens



from toiro import classifiers, datadownloader


# def test_classifier_svm():
#     # Download the livedoor news corpus and load it as pandas.DataFrame
#     corpora = datadownloader.available_corpus()
#     livedoor_corpus = corpora[0]
#     datadownloader.download_corpus(livedoor_corpus)
#     train_df, dev_df, test_df = datadownloader.load_corpus(
#         corpus=livedoor_corpus
#     )
#
#     model = classifiers.SVMClassificationModel()
#     model.fit(train_df, dev_df)
#     eval_result = model.eval(test_df)
#     print(eval_result)
#     print(eval_result["accuracy_score"])
#     print(eval_result["elapsed_time"])
#
#     model.save(f"{livedoor_corpus}.pkl")
#     model = classifiers.SVMClassificationModel(
#         model_file=f"{livedoor_corpus}.pkl"
#     )
#
#     text = "Python で前処理を"
#     pred_y = model.predict(text)
#
#     expected = "dokujo-tsushin"
#     assert pred_y == expected
#
#
# def test_classifier_bert():
#     if classifiers.is_bert_available():
#         train_df = classifiers.read_file(
#             datadownloader.sample_datasets.sample_train
#         )
#
#         dev_df = classifiers.read_file(
#             datadownloader.sample_datasets.sample_dev
#         )
#
#         # test_df = classifiers.read_file(
#         #     datadownloader.sample_datasets.sample_test
#         # )
#
#         model = classifiers.BERTClassificationModel()
#         model.fit(train_df, dev_df)
#
#         text = "Python で前処理を"
#         pred_y = model.predict(text)
#     else:
#         assert classifiers.is_bert_available() is False



import os
import sys

import sentencepiece as spm

base = os.path.dirname(os.path.abspath(__file__))
sys.path.append(base)


sp = spm.SentencePieceProcessor()

sp_model_file = base + "/data/ja.text8.txt.spm"
sp.load(f"{sp_model_file}.model")


def tokenize(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """
    words = sp.encode_as_pieces(text)
    return words


def original_usage(text):
    """
    Return the analysis results by sentencepiece.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : list
        The analysis results by sentencepiece
    """

    pieces = sp.encode_as_pieces(text)
    ids = sp.encode_as_ids(text)
    tokens = [pieces, ids]
    return tokens



import Mykytea

mk = Mykytea.Mykytea("")


def tokenize(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """

    words = [word for word in mk.getWS(text)]
    return words


def original_usage(text):
    """
    Return the analysis results by kytea.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : Mykytea.TagsVector
        The analysis results by kytea
    """

    tokens = mk.getAllTags(text)
    return tokens



import os
import csv
import json
import glob
import gzip
import random
import tarfile
import zipfile

import pandas as pd

from .downloader_utils import get_corpora_dict
from .downloader_utils import get_resource_dir


def _extract_tarfile(filename, target_dir):
    with tarfile.open(filename, 'r:*') as tar:
        tar.extractall(target_dir)


def _shuffle_data(is_shuffle, data):
    if is_shuffle:
        random.shuffle(data)


def _max_count_data(max_count, data):
    return data[:max_count]


def _split_train_dev_test(data, train_data=0.8, dev_data=0.1, test_data=0.1):
    total = train_data+dev_data+test_data
    if not total == 1.0:
        err_msg = f"The total of train/dev/test data: {total} must be 1."
        raise Exception(err_msg)

    num_train = int(len(data) * train_data)
    num_dev = int(len(data) * dev_data)
    num_test = int(len(data) * test_data)

    train = pd.DataFrame(data[:num_train])
    dev = pd.DataFrame(data[num_train:num_train+num_dev])
    test = pd.DataFrame(data[num_train+num_dev:num_train+num_dev+num_test])
    return train, dev, test


def _check_correct_corpus_type(corpus_type, corpus_types):
    if corpus_type not in corpus_types:
        err_msg = f"{corpus_type} is not available. Choose from {corpus_types}"
        raise Exception(err_msg)


def load_corpus(corpus, n=None, is_shuffle=True, corpus_type=None,
                train_data=0.8, dev_data=0.1, test_data=0.1, random_seed=1234):
    """
    Dataloader for selected corpus.

    The data is pre-processed and split into training data,
    development data and test data.

    Parameters
    ----------
    corpus : str
        The corpus

    n : int
        The number of datasets

    is_shuffle : bool
        If true, shuffle the dataset

    train_data : float
        Percentage of training data

    dev_data : float
        Percentage of development data

    test_data : float
        Percentage of test data

    random_seed : int
        Random seed for shuffle datasets

    Returns
    -------
    train_df : pandas.core.frame.DataFrame
        The training data

    dev_df : pandas.core.frame.DataFrame
        The development data

    test_df : pandas.core.frame.DataFrame
        The test data

    Examples
    --------
    >>> train_df, dev_df, test_df = datadownloader.load_corpus('livedoor_news_corpus')

    """

    if corpus == "amazon_reviews":
        return load_amazon_reviews(
            n=n, is_shuffle=is_shuffle,
            train_data=train_data, dev_data=dev_data, test_data=test_data,
            random_seed=random_seed)
    elif corpus == "yahoo_movie_reviews":
        if corpus_type is None:
            corpus_type = "binary"
        return load_yahoo_movie_reviews(
            n=n, is_shuffle=is_shuffle, corpus_type=corpus_type,
            train_data=train_data, dev_data=dev_data, test_data=test_data,
            random_seed=random_seed)
    elif corpus == "livedoor_news_corpus":
        if corpus_type is None:
            corpus_type = "title"
        return load_livedoor_news_corpus(
            n=n, is_shuffle=is_shuffle, corpus_type=corpus_type,
            train_data=train_data, dev_data=dev_data, test_data=test_data,
            random_seed=random_seed)
    elif corpus == "chABSA_dataset":
        return load_chABSA_dataset(
            n=n, is_shuffle=is_shuffle,
            train_data=train_data, dev_data=dev_data, test_data=test_data,
            random_seed=random_seed)
    else:
        err_msg = " ".join(
            [f"{corpus} does not exist.",
            f"Use datadownloader.download_corpus('{corpus}') ."]
        )
        raise Exception(err_msg)


def __count_polarity(opinions):
    posinega = {"positive": 1, "negative": -1}
    scores = [posinega.get(opinion["polarity"], 0) for opinion in opinions]
    score = sum(scores)

    if score > 0:
        return "positive"
    elif score < 0:
        return "negative"
    else:
        return "neutral"


def load_chABSA_dataset(n=None, is_shuffle=True,
                        train_data=0.8, dev_data=0.1, test_data=0.1,
                        random_seed=1234):
    """
    Dataloader for chABSA dataset.

    The data is pre-processed and split into training data,
    development data and test data.

    Parameters
    ----------
    n : int
        The number of datasets

    is_shuffle : bool
        If true, shuffle the dataset

    train_data : float
        Percentage of training data

    dev_data : float
        Percentage of development data

    test_data : float
        Percentage of test data

    random_seed : int
        Random seed for shuffle datasets

    Returns
    -------
    train_df : pandas.core.frame.DataFrame
        The training data

    dev_df : pandas.core.frame.DataFrame
        The development data

    test_df : pandas.core.frame.DataFrame
        The test data

    """
    random.seed(random_seed)

    corpus = "chABSA_dataset"
    corpora_dict = get_corpora_dict()
    resource_dir = get_resource_dir()

    filename = corpora_dict[corpus]["filename"]
    filepath = os.path.join(resource_dir, filename)

    with zipfile.ZipFile(filepath, 'r') as f:
        f.extractall(resource_dir)

    files = glob.glob(os.path.join(resource_dir, "chABSA-dataset", "*.json"))
    data = []
    for _file in files:
        with open(_file, "r") as f:
            _data = json.load(f)
            sentences = _data["sentences"]
            for sentence in sentences:
                sent = sentence["sentence"]
                opinions = sentence["opinions"]
                label = __count_polarity(opinions)
                data.append([label, sent])

    _shuffle_data(is_shuffle, data)
    data = _max_count_data(n, data)
    train_df, dev_df, test_df = _split_train_dev_test(
        data, train_data=train_data, dev_data=dev_data, test_data=test_data
    )
    return train_df, dev_df, test_df


def load_amazon_reviews(n=None, is_shuffle=True,
                        train_data=0.8, dev_data=0.1, test_data=0.1,
                        random_seed=1234):
    """
    Dataloader for amazon reviews.

    The data is pre-processed and split into training data,
    development data and test data.

    Parameters
    ----------
    n : int
        The number of datasets

    is_shuffle : bool
        If true, shuffle the dataset

    train_data : float
        Percentage of training data

    dev_data : float
        Percentage of development data

    test_data : float
        Percentage of test data

    random_seed : int
        Random seed for shuffle datasets

    Returns
    -------
    train_df : pandas.core.frame.DataFrame
        The training data

    dev_df : pandas.core.frame.DataFrame
        The development data

    test_df : pandas.core.frame.DataFrame
        The test data

    """

    random.seed(random_seed)

    corpus = "amazon_reviews"
    corpora_dict = get_corpora_dict()
    resource_dir = get_resource_dir()

    filename = corpora_dict[corpus]["filename"]
    filepath = os.path.join(resource_dir, filename)

    data = []
    if os.path.exists(filepath):
        with gzip.open(filepath, "rt") as f:
            reader = csv.reader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
            next(reader)
            for line in reader:
                rating = line[7]
                text = line[13]
                data.append([rating, text])

    _shuffle_data(is_shuffle, data)
    data = _max_count_data(n, data)
    train_df, dev_df, test_df = _split_train_dev_test(
        data, train_data=train_data, dev_data=dev_data, test_data=test_data
    )
    return train_df, dev_df, test_df


def load_yahoo_movie_reviews(n=None, is_shuffle=True, corpus_type="binary",
                             train_data=0.8, dev_data=0.1, test_data=0.1,
                             random_seed=1234):
    """
    Dataloader for yahoo_movie_reviews.

    The data is pre-processed and split into training data,
    development data and test data.

    Parameters
    ----------
    n : int
        The number of datasets

    is_shuffle : bool
        If true, shuffle the dataset

    train_data : float
        Percentage of training data

    dev_data : float
        Percentage of development data

    test_data : float
        Percentage of test data

    random_seed : int
        Random seed for shuffle datasets

    Returns
    -------
    train_df : pandas.core.frame.DataFrame
        The training data

    dev_df : pandas.core.frame.DataFrame
        The development data

    test_df : pandas.core.frame.DataFrame
        The test data

    """

    corpus_types = ["binary", "original"]
    _check_correct_corpus_type(corpus_type, corpus_types)
    random.seed(random_seed)

    corpus = "yahoo_movie_reviews"
    corpora_dict = get_corpora_dict()
    resource_dir = get_resource_dir()

    filename = corpora_dict[corpus]["filename"]
    filepath = os.path.join(resource_dir, filename)
    if os.path.exists(filepath):
        yahoo_movie_reviews_dir = os.path.join(resource_dir, "data")
        if not os.path.exists(yahoo_movie_reviews_dir):
            _extract_tarfile(filepath, resource_dir)

        yahoo_movie_reviews_json = os.path.join(
            yahoo_movie_reviews_dir, "yahoo-movie-reviews.json"
        )
        if not os.path.exists(yahoo_movie_reviews_json):
            err_msg = " ".join([
                f"{yahoo_movie_reviews_json} does not exist. ",
                f"Use datadownloader.download_corpus('{corpus}') ."
                ]
            )
            raise Exception(err_msg)

        data = []
        with open(yahoo_movie_reviews_json, "r") as f:
            json_load = json.load(f)
            for line in json_load:
                text = line["text"].replace("\n", "")
                rating = str(line["rating"])
                if corpus_type == "binary":
                    if rating in ["1", "2"]:
                        rating = 0
                        data.append([rating, text])
                    elif rating in ["4", "5"]:
                        rating = 1
                        data.append([rating, text])
                else:
                    data.append([rating, text])

        if corpus_type == "binary":
            label2texts = {}
            for line in data:
                label, text = line
                if label in label2texts:
                    label2texts[label].append(text)
                else:
                    label2texts[label] = [text]

            num_data = min(
                [len(label2texts[key]) for key in label2texts.keys()]
            )

            data = []
            for key in label2texts.keys():
                texts = label2texts[key][:num_data]
                for text in texts:
                    data.append([key, text])

        _shuffle_data(is_shuffle, data)
        data = _max_count_data(n, data)
        train_df, dev_df, test_df = _split_train_dev_test(
            data, train_data=train_data, dev_data=dev_data, test_data=test_data
        )
        return train_df, dev_df, test_df

    else:
        err_msg = " ".join(
            [f"{corpus} does not exist.",
            f"Use datadownloader.download_corpus('{corpus}') ."]
        )
        raise Exception(err_msg)


def load_livedoor_news_corpus(n=None, is_shuffle=True, corpus_type="title",
                              train_data=0.8, dev_data=0.1, test_data=0.1,
                              random_seed=1234):
    """
    Dataloader for livedoor news corpus.

    The data is pre-processed and split into training data,
    development data and test data.

    Parameters
    ----------
    n : int
        The number of datasets

    is_shuffle : bool
        If true, shuffle the dataset

    train_data : float
        Percentage of training data

    dev_data : float
        Percentage of development data

    test_data : float
        Percentage of test data

    random_seed : int
        Random seed for shuffle datasets

    Returns
    -------
    train_df : pandas.core.frame.DataFrame
        The training data

    dev_df : pandas.core.frame.DataFrame
        The development data

    test_df : pandas.core.frame.DataFrame
        The test data

    """

    corpus_types = ["title", "article"]
    _check_correct_corpus_type(corpus_type, corpus_types)

    random.seed(random_seed)
    corpus = "livedoor_news_corpus"
    label_names = [
        "dokujo-tsushin",
        "kaden-channel",
        "movie-enter",
        "smax",
        "topic-news",
        "it-life-hack",
        "livedoor-homme",
        "peachy",
        "sports-watch"
    ]

    corpora_dict = get_corpora_dict()
    resource_dir = get_resource_dir()

    filename = corpora_dict[corpus]["filename"]
    filepath = os.path.join(resource_dir, filename)

    if os.path.exists(filepath):
        livedoor_news_corpus_dir = os.path.join(resource_dir, "text")
        if not os.path.exists(livedoor_news_corpus_dir):
            _extract_tarfile(filepath, resource_dir)

        dirs = glob.glob(f"{livedoor_news_corpus_dir}/*")

        data = []
        for dir_name in dirs:
            dir_basename = os.path.basename(dir_name)
            if dir_basename in label_names:
                files = glob.glob(f"{dir_name}/*")
                for filename in files:
                    with open(filename, "r") as f:
                        article = []
                        for i, line in enumerate(f):
                            line = line.strip().replace("\t", "")
                            if corpus_type == "title":
                                if i == 2:
                                    data.append([dir_basename, line])

                            if corpus_type == "article":
                                if i > 2:
                                    article.append(line)

                    if corpus_type == "article":
                        article = "".join(article)
                        data.append([dir_basename, article])

        _shuffle_data(is_shuffle, data)
        data = _max_count_data(n, data)
        train_df, dev_df, test_df = _split_train_dev_test(
            data, train_data=train_data, dev_data=dev_data, test_data=test_data
        )
        return train_df, dev_df, test_df

    else:
        err_msg = " ".join(
            [f"{corpus} does not exist.",
            f"Use datadownloader.download_corpus('{corpus}') ."]
        )
        raise Exception(err_msg)



from toiro import tokenizers
from toiro import datadownloader

text = "Python で前処理を"


def test_janome():
    if tokenizers.is_janome_available():
        expected = ['Python', ' ', 'で', '前', '処理', 'を']
        words = tokenizers.tokenize_janome(text)
        assert words == expected

        tokens = tokenizers.original_janome(text)
        assert type(tokens) == list
    else:
        assert tokenizers.is_janome_available() is False


def test_nagisa():
    if tokenizers.is_nagisa_available():
        expected = ['Python', '\u3000', 'で', '前', '処理', 'を']
        words = tokenizers.tokenize_nagisa(text)
        assert words == expected

        expected = 'Python/名詞 \u3000/空白 で/助詞 前/名詞 処理/名詞 を/助詞'
        tokens = str(tokenizers.original_nagisa(text))
        assert tokens == expected
    else:
        assert tokenizers.is_nagisa_available() is False


def test_sudachipy():
    if tokenizers.is_sudachipy_available():
        expected = ['Python', ' ', 'で', '前処理', 'を']
        words = tokenizers.tokenize_sudachipy(text)
        assert words == expected

        tokens = tokenizers.original_sudachipy(text)
        expected = "<class 'sudachipy.morphemelist.MorphemeList'>"
        assert str(type(tokens)) == expected
    else:
        assert tokenizers.is_sudachipy_available() is False


def test_mecab():
    if tokenizers.is_mecab_available():
        expected = ['Python', 'で', '前', '処理', 'を']
        words = tokenizers.tokenize_mecab(text)
        assert words == expected

        tokens = tokenizers.original_mecab(text)
        assert type(tokens) == str
    else:
        assert tokenizers.is_mecab_available() is False


def test_spacy():
    if tokenizers.is_spacy_available():
        expected = ['Python', 'で', '前', '処理', 'を']
        words = tokenizers.tokenize_spacy(text)
        assert words == expected

        tokens = tokenizers.original_spacy(text)
        assert str(type(tokens)) == "<class 'spacy.tokens.doc.Doc'>"
    else:
        assert tokenizers.is_spacy_available() is False


def test_ginza():
    if tokenizers.is_ginza_available():
        expected = ['Python', 'で', '前処理', 'を']
        words = tokenizers.tokenize_ginza(text)
        assert words == expected

        tokens = tokenizers.original_ginza(text)
        assert str(type(tokens)) == "<class 'spacy.tokens.doc.Doc'>"
    else:
        assert tokenizers.is_ginza_available() is False


def test_kytea():
    if tokenizers.is_kytea_available():
        expected = ['Python', ' ', 'で', '前', '処理', 'を']
        words = tokenizers.tokenize_kytea(text)

        tokenizers.original_kytea(text)
        assert words == expected
    else:
        assert tokenizers.is_kytea_available() is False


def test_jumanpp():
    if tokenizers.is_jumanpp_available():
        expected = ['Python', '\u3000', 'で', '前', '処理', 'を']
        words = tokenizers.tokenize_jumanpp(text)
        assert words == expected

        tokens = tokenizers.original_jumanpp(text)
        assert str(type(tokens)) == "<class 'pyknp.juman.mlist.MList'>"
    else:
        assert tokenizers.is_jumanpp_available() is False


def test_sentencepiece():
    if tokenizers.is_sentencepiece_available():
        expected = ['▁', 'P', 'y', 'th', 'on', '▁', 'で', '前', '処理', 'を']
        words = tokenizers.tokenize_sentencepiece(text)
        assert words == expected

        expected = [
            ['▁', 'P', 'y', 'th', 'on', '▁', 'で', '前', '処理', 'を'],
            [5, 0, 210, 1040, 905, 5, 14, 144, 2828, 7]
        ]
        tokens = tokenizers.original_sentencepiece(text)
        assert tokens == expected
    else:
        assert tokenizers.is_sentencepiece_available() is False


def test_fugashi_ipadic():
    if tokenizers.is_fugashi_ipadic_available():
        expected = ['Python', 'で', '前', '処理', 'を']
        words = tokenizers.tokenize_fugashi_ipadic(text)
        assert words == expected

        tokens = tokenizers.original_fugashi_ipadic(text)
        assert type(tokens) == list
    else:
        tokenizers.is_fugashi_ipadic_available() is False


def test_tinysegmenter():
    if tokenizers.is_tinysegmenter_available():
        expected = ['Python', ' ', 'で', '前処', '理', 'を']
        words = tokenizers.tokenize_tinysegmenter(text)
        assert words == expected

        tokens = tokenizers.original_tinysegmenter(text)
        assert type(tokens) == list
    else:
        tokenizers.is_tinysegmenter_available() is False


def test_fugashi_unidic():
    if tokenizers.is_fugashi_unidic_available():
        expected = ['Python', 'で', '前', '処理', 'を']
        words = tokenizers.tokenize_fugashi_unidic(text)
        assert words == expected

        tokens = tokenizers.original_fugashi_unidic(text)
        assert type(tokens) == list
    else:
        tokenizers.is_fugashi_unidic_available() is False


def test_tiktoken():
    if tokenizers.is_tiktoken_available():
        expected_gpt4o = ['Python', ' ', 'で', '前', '処', '理', 'を']
        words_gpt4o = tokenizers.tokenize_tiktoken_gpt4o(text)
        assert words_gpt4o == expected_gpt4o

        tokens_gpt4o = tokenizers.original_tiktoken_gpt4o(text)
        assert type(tokens_gpt4o) == list

        expected_gpt5 = ['Python', ' ', 'で', '前', '処', '理', 'を']
        words_gpt5 = tokenizers.tokenize_tiktoken_gpt5(text)
        assert words_gpt5 == expected_gpt5

        tokens_gpt5 = tokenizers.original_tiktoken_gpt5(text)
        assert type(tokens_gpt5) == list

    else:
        tokenizers.is_tiktoken_available() is False



def test_compare():
    texts = [text]
    report = tokenizers.compare(texts)
    assert type(report) == dict


def test_compare_from_file():
    filename = datadownloader.sample_datasets.sample_txt
    report = tokenizers.compare_from_file(filename)
    print(report)
    assert type(report) == dict


def test_compare_from_file_with_disable_tokenizers():
    filename = datadownloader.sample_datasets.sample_txt
    report = tokenizers.compare_from_file(
        filename, disable_tokenizers=["nagisa"]
    )
    print(report)
    assert type(report) == dict


def test_print_words():
    tokenizers.print_words(text)


def test_print_words_with_disable_tokenizers():
    tokenizers.print_words(text, disable_tokenizers=["nagisa"])


def char_tokenize(text):
    chars = list(text)
    return chars


def test_compare_with_additional_tokenizers():
    texts = [text]

    additional_tokenizers = {"char": char_tokenize}

    report = tokenizers.compare(
        texts, additional_tokenizers=additional_tokenizers
    )
    assert type(report) == dict


def test_compare_from_file_with_additional_tokenizers():
    filename = datadownloader.sample_datasets.sample_txt

    additional_tokenizers = {"char": char_tokenize}
    report = tokenizers.compare_from_file(
        filename, additional_tokenizers=additional_tokenizers
    )
    print(report)
    assert type(report) == dict


def test_print_words_with_additional_tokenizers():
    additional_tokenizers = {"char": char_tokenize}
    tokenizers.print_words(text, additional_tokenizers=additional_tokenizers)



import time

from tqdm import tqdm
from cpuinfo import get_cpu_info

import numpy as np
from toiro import tokenizers


def get_avaiable_tokenizers(disable_tokenizers=None):
    if disable_tokenizers is None:
        disable_tokenizers = []

    _tokenizers = {}
    if tokenizers.is_mecab_available():
        _mecab_python3 = 'mecab-python3'
        if _mecab_python3 not in disable_tokenizers:
            _tokenizers[_mecab_python3] = tokenizers.tokenize_mecab

    if tokenizers.is_janome_available():
        _janome = 'janome'
        if _janome not in disable_tokenizers:
            _tokenizers[_janome] = tokenizers.tokenize_janome

    if tokenizers.is_nagisa_available():
        _nagisa = 'nagisa'
        if _nagisa not in disable_tokenizers:
            _tokenizers[_nagisa] = tokenizers.tokenize_nagisa

    if tokenizers.is_sudachipy_available():
        _sudachipy = 'sudachipy'
        if _sudachipy not in disable_tokenizers:
            _tokenizers[_sudachipy] = tokenizers.tokenize_sudachipy

    if tokenizers.is_spacy_available():
        _spacy = 'spacy'
        if _spacy not in disable_tokenizers:
            _tokenizers[_spacy] = tokenizers.tokenize_spacy

    if tokenizers.is_ginza_available():
        _ginza = 'ginza'
        if _ginza not in disable_tokenizers:
            _tokenizers[_ginza] = tokenizers.tokenize_ginza

    if tokenizers.is_kytea_available():
        _kytea = 'kytea'
        if _kytea not in disable_tokenizers:
            _tokenizers[_kytea] = tokenizers.tokenize_kytea

    if tokenizers.is_jumanpp_available():
        _jumanpp = 'jumanpp'
        if _jumanpp not in disable_tokenizers:
            _tokenizers[_jumanpp] = tokenizers.tokenize_jumanpp

    if tokenizers.is_sentencepiece_available():
        _sentencepiece = 'sentencepiece'
        if _sentencepiece not in disable_tokenizers:
            _tokenizers[_sentencepiece] = tokenizers.tokenize_sentencepiece

    if tokenizers.is_fugashi_ipadic_available():
        _fugashi_ipadic = 'fugashi-ipadic'
        if _fugashi_ipadic not in disable_tokenizers:
            _tokenizers[_fugashi_ipadic] = tokenizers.tokenize_fugashi_ipadic

    if tokenizers.is_fugashi_unidic_available():
        _fugashi_unidic = 'fugashi-unidic'
        if _fugashi_unidic not in disable_tokenizers:
            _tokenizers[_fugashi_unidic] = tokenizers.tokenize_fugashi_unidic

    if tokenizers.is_tinysegmenter_available():
        _tinysegmenter = 'tinysegmenter'
        if _tinysegmenter not in disable_tokenizers:
            _tokenizers[_tinysegmenter] = tokenizers.tokenize_tinysegmenter

    if tokenizers.is_tiktoken_available():
        _tiktoken_gpt4o = 'tiktoken_gpt4o'
        if _tiktoken_gpt4o not in disable_tokenizers:
            _tokenizers[_tiktoken_gpt4o] = tokenizers.tokenize_tiktoken_gpt4o

        _tiktoken_gpt5 = 'tiktoken_gpt5'
        if _tiktoken_gpt5 not in disable_tokenizers:
            _tokenizers[_tiktoken_gpt5] = tokenizers.tokenize_tiktoken_gpt5

    return _tokenizers


class SelectTokenizer:

    def __init__(self, tokenizer):
        avaiable_tokenizers = get_avaiable_tokenizers()
        self.tokenize = avaiable_tokenizers[tokenizer]


def _make_initial_report(texts):
    report = {'execution_environment': {}}
    cpu_info = get_cpu_info()
    keys = ['python_version', 'arch', 'brand_raw', 'count']
    for key in keys:
        report['execution_environment'][key] = cpu_info[key]

    report['data'] = {'number_of_sentences': len(texts)}
    average_length = np.average([len(text) for text in texts])
    report['data']['average_length'] = average_length
    return report


def compare_from_file(filename, disable_tokenizers=None, verbose=True,
                      additional_tokenizers=None):
    """
    Method to compare the tokenizers from an input text.

    This function outputs information about the execution environment
    and data and comparison results.

    Parameters
    ----------
    filename : str
        An input file

    disable_tokenizers : list
        A list of non-comparative tokenizers

    verbose : bool
        If True, show the process

    additional_tokenizers: dict
        A list of tokenizers to be added to the comparison is
        in dictionary form,
        where the keys are names and the values are functions.

    Returns
    -------
    report : dict
        Comparison results

    Examples
    --------
    >>> report = tokenizers.corpora_from_file(filename)
    >>> print(report)
    """
    with open(filename, "r") as f:
        texts = [line.strip() for line in f if len(line.strip()) > 0]
    report = compare(
        texts, disable_tokenizers=disable_tokenizers, verbose=verbose,
        additional_tokenizers=additional_tokenizers
    )
    return report


def compare(texts, disable_tokenizers=None, verbose=True,
            additional_tokenizers=None):
    """
    Method to compare the tokenizers for a list of input text.

    This function outputs information about the execution environment
    and data and comparison results.

    Parameters
    ----------
    texts : list
        A list of input text

    disable_tokenizers : list
        A list of non-comparative tokenizers

    verbose : bool
        If True, show the process

    additional_tokenizers: dict
        A list of tokenizers to be added to the comparison is
        in dictionary form,
        where the keys are names and the values are functions.

    Returns
    -------
    report : dict
        Comparison results

    Examples
    --------
    >>> report = tokenizers.corpora(texts)
    >>> print(report)
    """
    report = _make_initial_report(texts)
    tokenizers = get_avaiable_tokenizers(disable_tokenizers)
    if additional_tokenizers:
        tokenizers.update(additional_tokenizers)

    num_tokenizers = len(tokenizers.items())
    for i, (tokenizer_name, tokenize) in enumerate(tokenizers.items(), 1):
        if verbose:
            print(f"[{i}/{num_tokenizers}] Tokenizer: {tokenizer_name}")
            texts = tqdm(texts)

        start = time.time()
        for text in texts:
            words = tokenize(text)
        elapsed_time = time.time() - start

        report[tokenizer_name] = {}
        report[tokenizer_name]['elapsed_time'] = elapsed_time

    return report


def print_words(text, disable_tokenizers=None, delimiter=" ",
                additional_tokenizers=None):
    """
    Method to compare the segmented words.

    Print the results of the compared word segmentation.

    Parameters
    ----------
    texts : str
        An input text

    disable_tokenizers : list
        A list of non-comparative tokenizers

    delimiter : str
        A delimiter of words

    additional_tokenizers: dict
        A list of tokenizers to be added to the comparison is
        in dictionary form,
        where the keys are names and the values are functions.

    Examples
    --------
    >>> text = "単語分割の結果を比較します"
    >>> tokenizers.print_words(text, delimiter="|")
           nagisa: 単語|分割|の|結果|を|比較|し|ます
           janome: 単語|分割|の|結果|を|比較|し|ます
    mecab_python3: 単語|分割|の|結果|を|比較|し|ます
    """
    tokenizers = get_avaiable_tokenizers(disable_tokenizers=disable_tokenizers)
    if additional_tokenizers:
        tokenizers.update(additional_tokenizers)

    for tokenizer_name, tokenize in tokenizers.items():
        words = tokenize(text)
        words = delimiter.join(words)
        print(f"{tokenizer_name:>14}: {words}")



from setuptools import setup

classifiers = [
    'License :: OSI Approved :: MIT License', 'Natural Language :: Japanese',
    'Programming Language :: Python :: 3.10',
    'Programming Language :: Python :: 3.11',
    'Programming Language :: Python :: 3.12',
    'Programming Language :: Python :: 3.13',
    'Operating System :: Unix',
    'Topic :: Text Processing :: Linguistic',
    'Topic :: Software Development :: Libraries :: Python Modules'
]


def _long_description():
    readme = 'README.md'
    with open(readme, 'r') as f:
        long_description = f.read()
    return long_description


extras_requires_tokenizers = [
    'nagisa',
    'sudachipy',
    'sudachidict_core',
    'mecab-python3',
    'spacy',
    'ginza',
    'kytea',
    'pyknp',
    'sentencepiece',
    'fugashi',
    'ipadic',
    'unidic-lite',
    'tinysegmenter3',
]

extras_requires_classifiers = ['torch', 'transformers', 'catalyst']

setup(
    name='toiro',
    version='0.0.11',
    description='A comparison tool of Japanese tokenizers',
    author='Taishi Ikeda',
    author_email='taishi.ikeda.0323@gmail.com',
    long_description_content_type='text/markdown',
    long_description=_long_description(),
    keywords='Japanese NLP',
    url='https://github.com/taishi-i/toiro',
    download_url='https://github.com/taishi-i/toiro/archive/0.0.11.tar.gz',
    packages=['toiro.datadownloader', 'toiro.tokenizers', 'toiro.classifiers'],
    install_requires=[
        'requests', 'tqdm', 'pandas', 'scikit-learn', 'py-cpuinfo', 'janome'
    ],
    extras_require={
        'all_tokenizers': extras_requires_tokenizers,
        'all_classifiers': extras_requires_classifiers,
        'all': extras_requires_tokenizers + extras_requires_classifiers
    },
    classifiers=classifiers,
    include_package_data=True,
    python_requires='>=3.10.0',
    setup_requires=['pytest-runner'],
    tests_require=['pytest', 'pytest-cov'])



import fugashi

tagger = fugashi.Tagger()


def tokenize(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """

    words = [word.surface for word in tagger(text)]
    return words


def original_usage(text):
    """
    Return the analysis results by fugashi with unidic-lite.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : list
        A list of fugashi.fugashi.Node
    """

    tokens = tagger(text)
    return tokens



import tinysegmenter


def tokenize(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """

    words = tinysegmenter.tokenize(text)
    return words


def original_usage(text):
    """
    Return the analysis results by tinysegmenter.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : list
        A list of words
    """
    tokens = tinysegmenter.tokenize(text)
    return tokens



import os
from pathlib import Path

_LIB_NAME = "toiro"

_HOME_DIR = str(Path.home())
_RESOURCE_DIR = os.getenv(
    f"{_LIB_NAME}", os.path.join(_HOME_DIR, f'{_LIB_NAME}_resources')
)

# A dataset list
_CORPORA_DICT = {
    "livedoor_news_corpus": {
        "url": "https://www.rondhuit.com/download/ldcc-20140209.tar.gz",
        "filename": "ldcc-20140209.tar.gz"
    },
    "yahoo_movie_reviews":
        {
            "url": "https://media.githubusercontent.com/media/dennybritz/sentiment-analysis/master/data/yahoo-movie-reviews.json.tar.gz",
            "filename": "yahoo-movie-reviews.json.tar.gz"
        },

    "amazon_reviews":
        {
            "url": "https://s3.amazonaws.com/amazon-reviews-pds/tsv/amazon_reviews_multilingual_JP_v1_00.tsv.gz",
            "filename": "amazon_reviews_multilingual_JP_v1_00.tsv.gz"
        },
    "chABSA_dataset":
        {
            "url": "https://s3-ap-northeast-1.amazonaws.com/dev.tech-sketch.jp/chakki/public/chABSA-dataset.zip",
            "filename": "chABSA-dataset.zip"
        }
}


def get_resource_dir():
    """
    Return the path of the toiro directory.

    The dataset directory of this tool is created
    as toiro_resources in your home directory.

    Returns
    -------
    _RESOURCE_DIR : str
        The toiro directory in your execution environment.

    Examples
    --------
    >>> toiro.get_resource_dir()
    '~/toiro_resources'

    """
    return _RESOURCE_DIR


def get_corpora_dict():
    """
    Return the dataset information in toiro.

    Returns
    -------
    _CORPORA_DICT : dict
        The information of the avaibale corpus in this tool.

    Examples
    --------
    >>> toiro.get_corpora_dict()

    """
    return _CORPORA_DICT



import os

import pytest

from toiro import datadownloader


def test_available_corpus():
    corpora = datadownloader.available_corpus()
    excepted = [
        'livedoor_news_corpus', 'yahoo_movie_reviews',
        'amazon_reviews', 'chABSA_dataset'
    ]
    assert corpora == excepted


def test_check_correct_corpus_type_error():
    with pytest.raises(Exception):
        corpus = ""
        datadownloader.download_corpus(corpus=corpus)


# def test_download_corpus():
#     available_corpus = datadownloader.available_corpus()
#     for corpus in available_corpus:
#         datadownloader.download_corpus(corpus)
#
#         corpora_dict = datadownloader.get_corpora_dict()
#         resource_dir = datadownloader.get_resource_dir()
#
#         filename = corpora_dict[corpus]['filename']
#         filepath = os.path.join(resource_dir, filename)
#         assert os.path.exists(filepath)


def test_split_train_dev_test_error():
    corpora = datadownloader.available_corpus()
    corpus = corpora[0]
    with pytest.raises(Exception):
        train_data = 0.9
        dev_data = 0.2
        test_data = 0.1
        train_df, dev_df, test_df = datadownloader.load_corpus(
            corpus=corpus,
            train_data=train_data, dev_data=dev_data, test_data=test_data
        )


# def test_load_corpus():
#     available_corpus = datadownloader.available_corpus()
#
#     num_corpus = {
#         'livedoor_news_corpus': {'train': 5900, 'dev': 737, 'test': 737},
#         'yahoo_movie_reviews': {'train': 72956, 'dev': 9119, 'test': 9119},
#         'amazon_reviews': {'train': 209944, 'dev': 26243, 'test': 26243},
#         'chABSA_dataset': {'train': 4895, 'dev': 611, 'test': 611}
#     }
#
#     for corpus in available_corpus:
#         if corpus == 'livedoor_news_corpus':
#             train_df, dev_df, test_df = datadownloader.load_corpus(
#                 corpus=corpus
#             )
#
#         elif corpus == 'yahoo_movie_reviews':
#             train_df, dev_df, test_df = datadownloader.load_corpus(
#                 corpus=corpus, corpus_type='original'
#             )
#
#         elif corpus == 'amazon_reviews':
#             train_df, dev_df, test_df = datadownloader.load_corpus(
#                 corpus=corpus
#             )
#         elif corpus == 'chABSA_dataset':
#             train_df, dev_df, test_df = datadownloader.load_corpus(
#                 corpus=corpus
#             )
#
#         num_data = num_corpus[corpus]
#         excepted_train = num_data['train']
#         excepted_dev = num_data['dev']
#         excepted_test = num_data['test']
#
#         assert len(train_df) == excepted_train
#         assert len(dev_df) == excepted_dev
#         assert len(test_df) == excepted_test



from .classifier_utils import (
    read_file,
    EvaluationMetrics
)

from .classifier_svm import SVMClassificationModel


try:
    import torch
    import transformers
    import catalyst
    _bert_available = True
except:
    _bert_available = False


def is_bert_available():
    return _bert_available


if is_bert_available():
    from .classifier_bert import BERTClassificationModel



import os

import ipadic
import fugashi

dic_dir = ipadic.DICDIR
mecabrc = os.path.join(dic_dir, "mecabrc")
mecab_option = f"-d {dic_dir} -r {mecabrc}"
tagger = fugashi.GenericTagger(mecab_option)


def tokenize(text):
    """
    A method for word segmentation.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    words : list
        A list of words
    """

    words = [word.surface for word in tagger(text)]
    return words


def original_usage(text):
    """
    Return the analysis results by fugashi with ipadic.

    Parameters
    ----------
    text : str
        An input text

    Returns
    -------
    tokens : list
        A list of fugashi.fugashi.Node
    """
    # tokens = tokenize(text)
    tokens = tagger(text)
    return tokens



import os
import time
import pickle

import torch

from torch.utils.data import Dataset
from torch.utils.data import DataLoader

from transformers import AutoModel
from transformers import AutoConfig
from transformers import AutoTokenizer

from catalyst.dl import SupervisedRunner
from catalyst.dl.callbacks import AccuracyCallback

from sklearn.metrics import f1_score
from sklearn.metrics import accuracy_score
from sklearn.metrics import classification_report

from toiro import classifiers


class BERTBaseJapaneseModel(torch.nn.Module):
    def __init__(self, model_name, num_labels):
        super().__init__()
        config = AutoConfig.from_pretrained(model_name, num_labels=num_labels)
        self.bert = AutoModel.from_pretrained(model_name, config=config)
        self.pre_classifier = torch.nn.Linear(
            config.hidden_size, config.hidden_size
        )
        self.classifier = torch.nn.Linear(config.hidden_size, num_labels)
        seq_classification_dropout = 0.2
        self.dropout = torch.nn.Dropout(seq_classification_dropout)

    def forward(self, features, attention_mask=None, head_mask=None):
        assert attention_mask is not None, "attention mask is none"
        bert_output = self.bert(
            input_ids=features,
            attention_mask=attention_mask,
            head_mask=head_mask
        )
        hidden_state = bert_output[0]
        pooled_output = hidden_state[:, 0]
        pooled_output = self.pre_classifier(pooled_output)
        pooled_output = torch.nn.ReLU()(pooled_output)
        pooled_output = self.dropout(pooled_output)
        logits = self.classifier(pooled_output)
        return logits


class ClassificationDataset(Dataset):

    def __init__(self,
                 tokenizer, label2id, max_seq_length, texts, labels=None):
        self.tokenizer = tokenizer
        self.pad_vid = self.tokenizer.vocab["[PAD]"]
        self.label2id = label2id
        self.texts = texts
        self.labels = labels
        self.max_seq_length = max_seq_length

    def __len__(self):
        return len(self.texts)

    def __getitem__(self, index):
        text = self.texts[index]
        output_dict = self._from_text(text)
        if self.labels is not None:
            y = self.labels[index]
            y_encoded = torch.Tensor(
                [self.label2id.get(y, -1)]
            ).long().squeeze(0)
            output_dict["targets"] = y_encoded
        return output_dict

    def _from_text(self, text):
        x_encoded = self.tokenizer.encode(
            text,
            add_special_tokens=True,
            max_length=self.max_seq_length,
            return_tensors="pt",
            truncation=True
        ).squeeze(0)
        true_seq_length = x_encoded.size(0)
        pad_size = self.max_seq_length - true_seq_length
        pad_ids = torch.Tensor([self.pad_vid] * pad_size).long()
        x_tensor = torch.cat((x_encoded, pad_ids))
        mask = torch.ones_like(x_encoded, dtype=torch.int8)
        mask_pad = torch.zeros_like(pad_ids, dtype=torch.int8)
        mask = torch.cat((mask, mask_pad))
        output_dict = {"features": x_tensor, "attention_mask": mask}
        return output_dict


class BERTClassificationModel:

    def __init__(self,
                 model_name="cl-tohoku/bert-base-japanese-whole-word-masking",
                 checkpoints_dir=None):

        """
        Text classification model based on Japanese BERT Model.

        Attributes
        ----------
        model_name : str
            The BERT model file
        checkpoints_dir : str
            The path of trained BERT model dir

        -------
        fit()
            Train a text classification model.
        eval()
            Evaluate the trained model.
        predict()
            Predict a label.
        """

        self.runner = SupervisedRunner(
            input_key=("features", "attention_mask")
        )

        if checkpoints_dir:
            config_file = f"{checkpoints_dir}/checkpoints/config.pkl"
            if os.path.exists(config_file):
                with open(config_file, "rb") as f:
                    self.label2id, self.config = pickle.load(f)
                    self.id2label = {v: k for k, v in self.label2id.items()}

                num_labels = len(self.label2id)
                self.max_seq_length = self.config["max_seq_length"]
                self.batch_size = self.config["batch_size"]
                self.model_name = self.config["model_name"]
                self.elapsed_time = self.config["elapsed_time"]
                self.tokenizer = AutoTokenizer.from_pretrained(self.model_name)
                self.model = BERTBaseJapaneseModel(self.model_name, num_labels)

                self.data_for_predict = ClassificationDataset(
                    tokenizer=self.tokenizer,
                    label2id=self.label2id,
                    max_seq_length=self.max_seq_length,
                    texts=["checkpoints"]
                )

                temporary_data = {
                    "temporary": DataLoader(
                        dataset=self.data_for_predict,
                        batch_size=self.batch_size,
                        shuffle=False
                    )
                }

                # Load the trained BERT model
                self.runner.infer(
                    model=self.model,
                    loaders=temporary_data,
                    resume=f"{checkpoints_dir}/checkpoints/best.pth"
                )

        else:
            self.model_name = model_name
            self.tokenizer = AutoTokenizer.from_pretrained(model_name)
            self.pad_vid = self.tokenizer.vocab["[PAD]"]
            self.data_for_predict = None

    def fit(self,
            train_df, dev_df,
            batch_size=16, max_seq_length=256, learning_rate=5e-5,
            epochs=1, log_dir=None, verbose=False):

            start = time.time()
            config = {
                "model_name": self.model_name,
                "batch_size": batch_size,
                "max_seq_length": max_seq_length,
                "learning_rate": learning_rate,
                "epochs": epochs,
                "log_dir": log_dir
            }

            train_y = train_df[0]
            train_X = train_df[1]
            label2id = dict(
                zip(sorted(set(train_y)), range(len(set(train_y))))
            )
            self.id2label = {v: k for k, v in label2id.items()}
            num_labels = len(label2id)

            self.train_data = ClassificationDataset(
                tokenizer=self.tokenizer,
                label2id=label2id,
                max_seq_length=max_seq_length,
                texts=train_X,
                labels=train_y
            )

            dev_y = dev_df[0]
            dev_X = dev_df[1]

            self.dev_data = ClassificationDataset(
                tokenizer=self.tokenizer,
                label2id=label2id,
                max_seq_length=max_seq_length,
                texts=dev_X,
                labels=dev_y
            )

            train_dev_loaders = {
                "train": DataLoader(
                    dataset=self.train_data,
                    batch_size=batch_size,
                    shuffle=True
                ),
                "valid": DataLoader(
                    dataset=self.dev_data,
                    batch_size=batch_size,
                    shuffle=False
                )
            }

            model = BERTBaseJapaneseModel(self.model_name, num_labels)
            criterion = torch.nn.CrossEntropyLoss()
            optimizer = torch.optim.Adam(model.parameters(), lr=learning_rate)
            scheduler = torch.optim.lr_scheduler.ReduceLROnPlateau(optimizer)

            self.runner.train(
                model=model,
                criterion=criterion,
                optimizer=optimizer,
                scheduler=scheduler,
                loaders=train_dev_loaders,
                callbacks=[
                    AccuracyCallback(num_classes=num_labels),
                ],
                fp16=None,
                logdir=log_dir,
                num_epochs=epochs,
                verbose=verbose
            )

            self.elapsed_time = time.time() - start
            config["elapsed_time"] = self.elapsed_time

            if os.path.exists(f"{log_dir}/checkpoints"):
                filename = f"{log_dir}/checkpoints/config.pkl"
                with open(filename, "wb") as f:
                    pickle.dump([label2id, config], f)

    def predict(self, text):
        if self.data_for_predict:
            x = self.data_for_predict._from_text(text)
        else:
            x = self.train_data._from_text(text)

        x["features"] = x["features"].reshape(1, -1)
        x["attention_mask"] = x["attention_mask"].reshape(1, -1)
        logits = self.runner.predict_batch(x)['logits']
        pred_id = logits.argmax(axis=1)
        pred_y = self.id2label[int(pred_id)]
        return pred_y

    def eval(self, test_df):
        test_Y = test_df[0]
        pred_Y = [self.predict(text) for text in test_df[1]]

        accuracy = accuracy_score(test_Y, pred_Y)
        macro_f1 = f1_score(test_Y, pred_Y, average="macro")
        cr = classification_report(test_Y, pred_Y)

        eval_metrics = classifiers.EvaluationMetrics(
            accuracy, macro_f1, cr, self.elapsed_time
        )
        return eval_metrics



import sys
from importlib import metadata


try:
    import pkg_resources
except ImportError:
    pkg_resources = None

if sys.version_info < (3, 12):
    import imp
    if pkg_resources is not None:
        imp.reload(pkg_resources)
else:
    if pkg_resources is not None:
        import importlib
        importlib.reload(pkg_resources)
    else:
        import warnings
        warnings.warn("pkg_resources not available on this Python version; skipping reload")


try:
    import nagisa
    _nagisa_available = True
    _nagisa_version = nagisa.__version__
except:
    _nagisa_available = False
    _nagisa_version = False


try:
    from janome import version
    _janome_available = True
    _janome_version = version.JANOME_VERSION
except:
    _janome_available = False
    _janome_version = False


try:
    import MeCab
    _mecab_available = True
    _mecab_version = MeCab.VERSION
except:
    _mecab_available = False
    _mecab_version = False


try:
    import sudachipy
    from sudachipy import tokenizer
    from sudachipy import dictionary

    _sudachipy_available = True
    _sudachipy_version = sudachipy.__version__
except:
    _sudachipy_available = False
    _sudachipy_version = False


try:
    import spacy
    from spacy.lang.ja import Japanese
    nlp = Japanese()
    _spacy_available = True
    _spacy_version = spacy.__version__
except:
    _spacy_available = False
    _spacy_version = False


try:
    import spacy
    nlp = spacy.load('ja_ginza')
    _ginza_available = True

    _ginza_version = spacy.__version__
except:
    _ginza_available = False
    _ginza_version = False


try:
    import Mykytea
    mk = Mykytea.Mykytea("")
    _kytea_available = True
    _kytea_version = str(metadata.version("Mykytea"))
except:
    _kytea_available = False
    _kytea_version = False


try:
    from pyknp import Juman
    jumanpp = Juman()
    _jumanpp_available = True
    _jumanpp_version = str(metadata.version("pyknp"))
except:
    _jumanpp_available = False
    _jumanpp_version = False


try:
    import sentencepiece as spm
    _sentencepiece_available = True
    _sentencepiece_version = str(metadata.version("sentencepiece"))
except:
    _sentencepiece_available = False
    _sentencepiece_version = False


try:
    import ipadic
    import fugashi
    _fugashi_ipadic_available = True
    _fugashi_ipadic_version = str(metadata.version("fugashi"))
except:
    _fugashi_ipadic_available = False
    _fugashi_ipadic_version = False


try:
    import tinysegmenter
    _tinysegmenter_available = True
    _tinysegmenter_version = str(metadata.version("tinysegmenter"))
except:
    _tinysegmenter_available = False
    _tinysegmenter_version = False


try:
    import fugashi
    import unidic_lite
    _fugashi_unidic_available = True
    _fugashi_unidic_version = str(metadata.version("fugashi"))
except:
    _fugashi_unidic_available = False
    _fugashi_unidic_version = False


try:
    import tiktoken
    _tiktoken_available = True
    _tiktoken_version = str(metadata.version("tiktoken"))
except:
    _tiktoken_available = False
    _tiktoken_version = False


def is_mecab_available():
    """
    Check if the library is available.

    This function checks if mecab-python3 is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _mecab_available : bool
        If True, mecab is available in your environment.

    Examples
    --------
    >>> tokenizers.is_mecab_available()
    True

    """
    return _mecab_available


def is_nagisa_available():
    """
    Check if the library is available.

    This function checks if nagisa is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _nagisa_available : bool
        If True, nagisa is available in your environment.

    Examples
    --------
    >>> tokenizers.is_nagisa_available()
    True

    """
    return _nagisa_available


def is_janome_available():
    """
    Check if the library is available.

    This function checks if Janome is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _janome_available : bool
        If True, janome is available in your environment.

    Examples
    --------
    >>> tokenizers.is_janome_available()
    True

    """
    return _janome_available


def is_sudachipy_available():
    """
    Check if the library is available.

    This function checks if Janome is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _sudachipy_available : bool
        If True, sudachipy is available in your environment.

    Examples
    --------
    >>> tokenizers.is_sudachipy_available()
    True

    """
    return _sudachipy_available


def is_spacy_available():
    """
    Check if the library is available.

    This function checks if spacy is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _spacy_available : bool
        If True, spacy is available in your environment.

    Examples
    --------
    >>> tokenizers.is_spacy_available()
    True

    """
    return _spacy_available


def is_ginza_available():
    """
    Check if the library is available.

    This function checks if spacy is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _ginza_available : bool
        If True, ginza is available in your environment.

    Examples
    --------
    >>> tokenizers.is_ginza_available()
    True

    """
    return _ginza_available


def is_kytea_available():
    """
    Check if the library is available.

    This function checks if kytea is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _kytea_available : bool
        If True, kytea is available in your environment.

    Examples
    --------
    >>> tokenizers.is_kytea_available()
    True

    """
    return _kytea_available


def is_jumanpp_available():
    """
    Check if the library is available.

    This function checks if jumanpp is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _jumanpp_available : bool
        If True, Juman++ is available in your environment.

    Examples
    --------
    >>> tokenizers.is_jumanpp_available()
    True

    """
    return _jumanpp_available


def is_sentencepiece_available():
    """
    Check if the library is available.

    This function checks if sentencepiece is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _sentencepiece_available : bool
        If True, sentencepiece is available in your environment.

    Examples
    --------
    >>> tokenizers.is_sentencepiece_available()
    True

    """
    return _sentencepiece_available


def is_fugashi_ipadic_available():
    """
    Check if the library is available.

    This function checks if sentencepiece is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _fugashi_ipadic_available : bool
        If True, fugashi wiht ipadic is available in your environment.

    Examples
    --------
    >>> tokenizers.is_fugashi_ipadic_available()
    True

    """

    return _fugashi_ipadic_available


def is_tinysegmenter_available():
    """
    Check if the library is available.

    This function checks if tinysegmenter is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _tinysegmenter_available : bool
        If True, tinysegmenter is available in your environment.

    Examples
    --------
    >>> tokenizers.is_tinysegmenter_available()
    True

    """

    return _tinysegmenter_available


def is_fugashi_unidic_available():
    """
    Check if the library is available.

    This function checks if sentencepiece is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _fugashi_unidic_available : bool
        If True, fugashi wiht unidic is available in your environment.

    Examples
    --------
    >>> tokenizers.is_fugashi_unidic_available()
    True

    """

    return _fugashi_unidic_available


def is_tiktoken_available():
    """
    Check if the library is available.

    This function checks if sentencepiece is available in your environment
    and returns the result as a bool value.

    Returns
    -------
    _tiktoken_available : bool
        If True, tiktoken is available in your environment.

    Examples
    --------
    >>> tokenizers.is_tiktoken_available()
    True

    """

    return _tiktoken_available



def available_tokenizers():
    """
    Return a list of available libraries.

    This function returns a list of the libraries supported by this library.

    Returns
    -------
    available_tokenizers_dict : bool
         A list of available libraries as dictionary

    Examples
    --------
    >>> tokenizers.available_tokenizers()
    """
    available_tokenizers_dict = {
        "nagisa": {
            "is_available": is_nagisa_available(),
            "version": _nagisa_version
        },
        "janome": {
            "is_available": is_janome_available(),
            "version": _janome_version
        },
        "mecab-python3": {
            "is_available": is_mecab_available(),
            "version": _mecab_version
        },
        "sudachipy": {
            "is_available": is_sudachipy_available(),
            "version": _sudachipy_version
        },
        "spacy": {
            "is_available": is_spacy_available(),
            "version": _spacy_version
        },
        "ginza": {
            "is_available": is_ginza_available(),
            "version": _ginza_version
        },
        "kytea": {
            "is_available": is_kytea_available(),
            "version": _kytea_version
        },
        "jumanpp": {
            "is_available": is_jumanpp_available(),
            "version": _jumanpp_version
        },
        "sentencepiece": {
            "is_available": is_sentencepiece_available(),
            "version": _sentencepiece_version
        },
        "fugashi-ipadic": {
            "is_available": is_fugashi_ipadic_available(),
            "version": _fugashi_ipadic_version
        },
        "tinysegmenter": {
            "is_available": is_tinysegmenter_available(),
            "version": _tinysegmenter_version
        },
        "fugashi-unidic": {
            "is_available": is_fugashi_unidic_available(),
            "version": _fugashi_unidic_version
        },
        "tiktoken": {
            "is_available": is_tiktoken_available(),
            "version": _tiktoken_available
        }
    }
    return available_tokenizers_dict
